@@ -1,12 +1,4 @@
-"""
-    test_util_fileutil
-    ~~~~~~~~~~~~~~~~~~
-
-    Tests sphinx.util.fileutil functions.
-
-    :copyright: Copyright 2007-2019 by the Sphinx team, see AUTHORS.
-    :license: BSD, see LICENSE for details.
-"""
+"""Tests sphinx.util.fileutil functions."""
 
 from unittest import mock
 
@@ -19,7 +11,7 @@ class DummyTemplateLoader(BuiltinTemplateLoader):
         super().__init__()
         builder = mock.Mock()
         builder.config.templates_path = []
-        builder.app.translater = None
+        builder.app.translator = None
         self.init(builder)
 
 
@@ -33,7 +25,7 @@ def test_copy_asset_file(tempdir):
 
     copy_asset_file(src, dest)
     assert dest.exists()
-    assert src.text() == dest.text()
+    assert src.read_text(encoding='utf8') == dest.read_text(encoding='utf8')
 
     # copy template file
     src = (tempdir / 'asset.txt_t')
@@ -43,7 +35,7 @@ def test_copy_asset_file(tempdir):
     copy_asset_file(src, dest, {'var1': 'template'}, renderer)
     assert not dest.exists()
     assert (tempdir / 'output.txt').exists()
-    assert (tempdir / 'output.txt').text() == '# template data'
+    assert (tempdir / 'output.txt').read_text(encoding='utf8') == '# template data'
 
     # copy template file to subdir
     src = (tempdir / 'asset.txt_t')
@@ -53,7 +45,7 @@ def test_copy_asset_file(tempdir):
 
     copy_asset_file(src, subdir1, {'var1': 'template'}, renderer)
     assert (subdir1 / 'asset.txt').exists()
-    assert (subdir1 / 'asset.txt').text() == '# template data'
+    assert (subdir1 / 'asset.txt').read_text(encoding='utf8') == '# template data'
 
     # copy template file without context
     src = (tempdir / 'asset.txt_t')
@@ -63,7 +55,7 @@ def test_copy_asset_file(tempdir):
     copy_asset_file(src, subdir2)
     assert not (subdir2 / 'asset.txt').exists()
     assert (subdir2 / 'asset.txt_t').exists()
-    assert (subdir2 / 'asset.txt_t').text() == '# {{var1}} data'
+    assert (subdir2 / 'asset.txt_t').read_text(encoding='utf8') == '# {{var1}} data'
 
 
 def test_copy_asset(tempdir):
@@ -72,13 +64,13 @@ def test_copy_asset(tempdir):
     # prepare source files
     source = (tempdir / 'source')
     source.makedirs()
-    (source / 'index.rst').write_text('index.rst')
-    (source / 'foo.rst_t').write_text('{{var1}}.rst')
+    (source / 'index.rst').write_text('index.rst', encoding='utf8')
+    (source / 'foo.rst_t').write_text('{{var1}}.rst', encoding='utf8')
     (source / '_static').makedirs()
-    (source / '_static' / 'basic.css').write_text('basic.css')
+    (source / '_static' / 'basic.css').write_text('basic.css', encoding='utf8')
     (source / '_templates').makedirs()
-    (source / '_templates' / 'layout.html').write_text('layout.html')
-    (source / '_templates' / 'sidebar.html_t').write_text('sidebar: {{var2}}')
+    (source / '_templates' / 'layout.html').write_text('layout.html', encoding='utf8')
+    (source / '_templates' / 'sidebar.html_t').write_text('sidebar: {{var2}}', encoding='utf8')
 
     # copy a single file
     assert not (tempdir / 'test1').exists()
@@ -91,11 +83,11 @@ def test_copy_asset(tempdir):
     copy_asset(source, destdir, context=dict(var1='bar', var2='baz'), renderer=renderer)
     assert (destdir / 'index.rst').exists()
     assert (destdir / 'foo.rst').exists()
-    assert (destdir / 'foo.rst').text() == 'bar.rst'
+    assert (destdir / 'foo.rst').read_text(encoding='utf8') == 'bar.rst'
     assert (destdir / '_static' / 'basic.css').exists()
     assert (destdir / '_templates' / 'layout.html').exists()
     assert (destdir / '_templates' / 'sidebar.html').exists()
-    assert (destdir / '_templates' / 'sidebar.html').text() == 'sidebar: baz'
+    assert (destdir / '_templates' / 'sidebar.html').read_text(encoding='utf8') == 'sidebar: baz'
 
     # copy with exclusion
     def excluded(path):
